@@ -1,8 +1,8 @@
 /*
  *  mp3info.h
  *  
- *  Author:
- *        Ricardo Cerqueira      <rmc@rccn.net>
+ * Original Author:
+ *       Ricardo Cerqueira      <rmc@rccn.net>
  *    
  *  This software is covered by the GNU public license, which should be in a file named
  *  LICENSE acompanying this.
@@ -18,6 +18,7 @@
 #include <signal.h>
 #include "textfunc.h"
 
-#define VERSION "MP3info - v0.5.0"
+#define VERSION "MP3info - v0.6.1"
 
-void tagedit_curs(FILE *fp, char *filename, char *line);
+void tagedit_curs(char *filename, int filenum, int fileoutof, struct id3tag *tag);
+void text_genre(unsigned char *genre,char *buffer);
